#/bin/bash

# Make and copy *.bin and *.hex to binhex folder.
# Created by dinau at 2014/10

# global value
# ex.
#	target=cq_myarm_lpc1114_gcc
#	os=ssp
#	checksum=1

# tool value
objcopy=arm-none-eabi-objcopy
gcc=arm-none-eabi-gcc
binhex_dir=target/$target/binhex

# build core function
function build_core() {
	# $1 : project name
	# $2 : project folder
	# $3 : checksum enable flag

	rm ./*
	echo

	version="$gcc --version"
	$version						> $1.log		2>&1

	conf="../configure -T $target -A $1 -t $2"
	echo $conf
	echo $conf						>> $1.log		2>&1
	$conf							>> $1.log		2>&1

	make depend						>> $1.log		2>&1
	make							>> $1.log		2>&1
	if [ $3 -ne 0 ] ; then
	checksum $os.bin				>> $1.log		2>&1
	fi
	cp $os.bin $1.bin
	$objcopy -I binary $1.bin -O ihex $1.hex

	# copy bin an hex to target folder
	cp $1.{bin,hex,log} ../$binhex_dir  
}

# build project with checksum 
function build_with_checksum() {
	# $1 : project name
	# $2 : project folder
	
	build_core $1 $2 1
}

# build project 
function build() {
	# $1 : project name
	# $2 : project folder
	
	build_core $1 $2 0
}

# warning message
function warning() {
echo 
echo
echo "[[[ Danger !! ]]]"
echo "All files in build folder are going to be deleted !"
echo " You have a chance to input [CTRL+C] key for breaking this script,"
echo " or input [Enter] key for continue script." 
read dummy
}

# init folder
function init_folder() {
	temp_folder=0temp
	mkdir $temp_folder
	mkdir $binhex_dir 
	cd $temp_folder
}

