/*
 *  TOPPERS/SSP Kernel
 *      Smallest Set Profile Kernel
 * 
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *  Copyright (C) 2005-2007 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 * 
 *  上記著作権者は，以下の(1)～(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 * 
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 * 
 */

/*
 * ターゲット依存モジュール（CQ-STARM用）
 */
#include "kernel_impl.h"
#include <sil.h>
#include "target_serial.h"
#include "target_syssvc.h"

#include "stm32f0xx.h"


/*
 * ターゲット依存部　初期化処理
 */
void target_initialize(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;

	SystemInit();

	/*
	 *  プロセッサ依存部の初期化
	 */
	prc_initialize();


	/*
	 *  ペリフェラルの有効化
	 */
	/* USART1(RX)   PA10 プルアップ */
	/* USART1(TX)  PA9 */

	RCC_AHBPeriphClockCmd( RCC_AHBPeriph_GPIOA, ENABLE);  
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1,ENABLE);

	GPIO_PinAFConfig( GPIOA, GPIO_PinSource10, GPIO_AF_1);
	GPIO_PinAFConfig( GPIOA, GPIO_PinSource9,  GPIO_AF_1);

	/* Configure UART1 pins:  Rx and Tx ----------------------------*/
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_AF;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_UP;

	GPIO_InitStructure.GPIO_Pin   =  GPIO_Pin_9;
	GPIO_Init( GPIOA, &GPIO_InitStructure); 
	GPIO_InitStructure.GPIO_Pin   = GPIO_Pin_10 ;
	GPIO_Init( GPIOA, &GPIO_InitStructure); 
#if (TNUM_PORT >= 2)
	/* USART2(RX) PA3 プルアップ */
	/* USART2(TX) PA2 */
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);  
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2,ENABLE);

	GPIO_PinAFConfig( GPIOA, GPIO_PinSource3, GPIO_AF_1);
	GPIO_PinAFConfig( GPIOA, GPIO_PinSource2, GPIO_AF_1);

	/* Configure UART2 pins:  Rx and Tx ----------------------------*/
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_AF;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_UP;

	GPIO_InitStructure.GPIO_Pin   =  GPIO_Pin_3;
	GPIO_Init( GPIOA, &GPIO_InitStructure); 
	GPIO_InitStructure.GPIO_Pin   =  GPIO_Pin_2;
	GPIO_Init( GPIOA, &GPIO_InitStructure); 
#endif

	/* LEDポート PC8, PC9*/
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOC, ENABLE);  

	/* Configure I/O pins:  PC8 and PC9 and Tx ----------------------------*/
	GPIO_InitStructure.GPIO_Pin   = GPIO_Pin_8 | GPIO_Pin_9;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_NOPULL;
	GPIO_Init( GPIOC, &GPIO_InitStructure); 

	/*
	 *  バナー出力用のシリアル初期化
	 */
	target_low_output_init(SIO_PORTID);
}

/*
 * ターゲット依存部 終了処理
 */
void target_exit(void)
{
	/* プロセッサ依存部の終了処理 */
	prc_terminate();
	
	while(true)
		;
}

/*
 * システムログの低レベル出力のための文字出力
 */
void target_fput_log(char_t c)
{
	if (c == '\n') {
		sio_pol_snd_chr('\r', SIO_PORTID);
	}
	sio_pol_snd_chr(c, SIO_PORTID);
}


